@@ -3,7 +3,7 @@ package CracTools::Utils;
   $CracTools::Utils::DIST = 'CracTools';
 }
 # ABSTRACT: A set of useful functions
-$CracTools::Utils::VERSION = '1.21';
+$CracTools::Utils::VERSION = '1.22';
 use strict;
 use warnings;
 
@@ -38,12 +38,42 @@ sub reverse_tab($) {
 }
 
 
+sub isVersionGreaterOrEqual($$) {
+  my ($v1,$v2) = @_;
+  my @v1_nums = split(/\./,$v1);
+  my @v2_nums = split(/\./,$v2);
+  for(my $i = 0; $i < @v1_nums; $i++) {
+    if($v1_nums[$i] >= $v2_nums[$i]) {
+      return 1;
+    }else {
+      return 0;
+    }
+  }
+  if(scalar @v2_nums > @v1_nums) {
+    return 0;
+  } else {
+    return 1;
+  }
+}
+
+
 my %conversion_hash = ( '+' => 1, '-' => '-1', '1' => '+', '-1' => '-');
 sub convertStrand($) {
   my $strand = shift;
   return $conversion_hash{$strand};
 }
 
+sub removeChrPrefix($) {
+  my $string = shift;
+  $string =~ s/^chr//;
+  return $string;
+}
+
+sub addChrPrefix($) {
+  my $string = shift;
+  return "chr".removeChrPrefix($string);
+}
+
 
 sub seqFileIterator {
   my ($file,$format) = @_;
@@ -185,8 +215,21 @@ sub bamFileIterator {
 
 sub getSeqFromIndexedRef {
   my ($ref_file,$chr,$pos,$length,$format) = @_;
-  my $region = "$chr:$pos-".($pos+$length-1);
-  return `samtools faidx $ref_file "$region"`;
+  $format = 'fasta' if !defined $format;
+  $pos++; # +1 because samtools faidx is 1-based
+  # First we try without the "chr" prefix
+  my $region = removeChrPrefix($chr).":$pos-".($pos+$length-1);
+  my $fasta_seq = `samtools faidx $ref_file "$region"`;
+  # If it has not worked, we try without
+  if($fasta_seq !~ /^>\S+\n\S+/) {
+    $region = addChrPrefix($chr).":$pos-".($pos+$length-1);
+    $fasta_seq = `samtools faidx $ref_file "$region"`;
+  }
+  if($format eq 'raw') {
+    my ($seq) = $fasta_seq =~ /^>\S+\n(\S+)/;
+    return $seq;
+  }
+  return $fasta_seq;
 }
 
 
@@ -468,7 +511,7 @@ CracTools::Utils - A set of useful functions
 
 =head1 VERSION
 
-version 1.21
+version 1.22
 
 =head1 SYNOPSIS
 
@@ -531,6 +574,10 @@ reverseComplement is more than B<100x faster than Bio-Perl> revcom_as_string()
   ReturnType  : String
   Exceptions  : none
 
+=head2 isVersionGreaterOrEqual($v1,$v2)
+
+Return true is version number v1 is greater than v2
+
 =head2 convertStrand
 
 Convert strand from '+/-' standard to '1/-1' standard and the opposite.
@@ -549,6 +596,22 @@ will print
   Reverse a: -1
   Reverse b: -
 
+=head2 removeChrPrefix
+
+Remove the "chr" prefix from a given string
+
+Example:
+
+  say "reference name: ",removeChrPrefix("chr1");
+
+will print
+
+  reference name: 1
+
+=head2 addChrPrefix
+
+Add the "chr" prefix to the given string
+
 =head1 PARSING
 
 This are some tools that aim to read (bio) files like
@@ -745,7 +808,8 @@ Return a sequence from a given region in a fasta indexed file
 
 Example:
 
-  my $fasta_seq = CracTools::Utils::getSeqFromIndexedRef("file.fa","chr2",29012,10);
+  my $fasta_seq = getSeqFromIndexedRef("file.fa","chr2",29012,10);
+  my $seq       = getSeqFromIndexedRef("file.fa","chr2",29012,10,'raw');
 
 =head1 PARSING LINES
 
